#! /usr/bin/python3
""" test.py
    
    Ce script permet de bloquer ou de débloquer des nodes connectés au Reflector Svxlink.
    Principales variables :
    links : liste des noms de nodes issus de svxreflector.log
    ips : liste des adresses IP des nodes.
    log : fichier intermédiaire. Extrait du log ne contenant que les Login.
    bip : dictionnary des nodes déjà bloqués.
"""
import curses
import curses.panel
import os
from datetime import datetime, date, time
import time

# constantes
SmallScreen = 0
MediumScreen = 1
LargeScreen = 2
TypeScreen = LargeScreen


links=list()
ips=list()
log=list()
bip=dict()
prov=dict()
logged=dict()
memeIP=list()

def fctSort(e):
    """ Cette fonction renvoie le nom du link afin de trier la liste. """
    n=e.split(":")
    return n[3].strip()


def getMemeIP(name,ip):
    memeIP.clear()
#    memeIP.append(name)
    for x in range(len(links)):
        if ips[x] == ip:
            if links[x] != name:
                memeIP.append(links[x])

def getNameBip(ip):
    for x,y in bip.items():
        if y == ip:
            return x
    return False

def clearWin2(w):
    if TypeScreen == LargeScreen:
        w.clear()
        w.border()
        Hw,Ww = w.getmaxyx()
        titre= " NODES DE MEME IP  "
        Xw= (Ww - len(titre))//2
        w.addstr(0,Xw,titre)
 
def fillWin2(w,name, ip, col):
    if TypeScreen == LargeScreen:
        clearWin2(w)
        ligne = 2
        getMemeIP(name,ip)
        w.addstr(ligne,2,"{} - {}".format(name,ip),col)
        ligne += 2
        for x in memeIP:
            w.addstr(ligne,2,x,col)
            ligne +=1
        w.refresh()

def wrline(w,h,n):
    """ La fonction wrline écrit une ligne à l'écran.
        Cette ligne comporte :
        - le nom du node
        - l'adresse ip sous laquelle le node s'est connecté la dernière fois
        - les caractères "<****>" pour prévenir que l'adresse a changé depuis le blocage
        - la mention 'blocked ' suivie de l'adresse ip lors du blocage.

        w= la fenêtre dans laquelle la fonction va écrire
        h= 0 écriture normale, 1 la ligne est surlignée
        n= le numéro de la ligne (= numéro d'ordre du node dans la liste)
    """
    name=links[n]
    if name in bip:
        color=curses.color_pair(2+h+h)
        if bip[name] != ips[n]:
           separ = "<****>"
        else:
           separ = "      "
        if TypeScreen == SmallScreen:
            w.addstr(n,0, "  {:<30}{:<18}".format(name,ips[n]),color)
        else:
            w.addstr(n,0, "  {:<30}{:<18}{:<8}bloqué :  {:<22}".format(name,ips[n],separ,bip[name]),color)
    elif ips[n] in bip.values():
        color=curses.color_pair(2+h+h)
        separ = "      "
        parName = getNameBip(ips[n])
        if TypeScreen == SmallScreen:
            w.addstr(n,0, "  {:<30}{:<18}".format(name,ips[n]),color)
        else:
            w.addstr(n,0, "  {:<30}{:<18}{:<8}bloqué par : {:<18} ".format(name,ips[n],separ,parName),color)
    else:
        color=curses.color_pair(1+h+h)
        if TypeScreen == SmallScreen:
            w.addstr(n,0, "  {:<30}{:<18}".format(name,ips[n]),color)
        else:
            w.addstr(n,0, "  {:<30}{:<58}".format(name, ips[n]),color)
#    if logged[name] == 1:
#        w.addstr(n,0,"_",curses.color_pair(7))


def readbip():
    """ readbip lit le fichier BIP.txt pour alimentre le dictionnary bip"""
    f=open("BIP.txt",'r')
    for x in f:
      e=x.split(":")
      bip[e[0]]=e[1]
    f.close()

def writebip():
    """ writebip sauvegarde le dictionnary bip dans BIP.txt."""
    f=open("BIP.txt",'w')
    for n in bip:
      txt=n+":"+bip[n]+':\n'
      f.write(txt)
    f.close()

def writelog(txt):
    fl=open("blockIP.log",'a')
    fl.write(txt)
    fl.close()

def addDrop(ip,name):
    """envoie la commande iptables pour bloquer IP"""
#    x="iptables -I INPUT -s "+ ip + " -p udp --dport 5300 -j DROP"
    x="iptables -I INPUT -s "+ ip + " -j DROP"
    os.system(x)
    d=datetime.today()
    txt=d.strftime("%d/%m/%Y %H:%M:%S")+";"+name+";"+ip+";blocage\n"
    writelog(txt)

def delDrop(ip,name):
    """envoie la commande iptables pour libérer IP"""
#    x="iptables -D INPUT -s "+ ip + " -p udp --dport 5300 --j DROP"
    x="iptables -D INPUT -s "+ ip + " -j DROP"
    os.system(x)
    d=datetime.today()
    txt=d.strftime("%d/%m/%Y %H:%M:%S")+";"+name+";"+ip+";déblocage\n"
    writelog(txt)

def commandes(scr):
    """liste des commandes en haut de l'écran."""
    global MW1
    scr.addstr(0,MW1+1, "flèches haut, bas : se déplacer dans la liste")
    scr.addstr(1,MW1+1, "F5 : bloquer le node surligné.")
    scr.addstr(2,MW1+1, "F6 : débloquer le node surligné.")
    scr.addstr(3,MW1+1, "ESC: quitter le programme.")


def readlog():
    """ lecture du svxreflector.log """
    
    f=open("/tmp/svxreflector.log")
    i=0
    for x in f:
        e=x.split(":")
        name = e[3].strip()
        if 'Login' in x:
            log.append(x)
            logged[name]=1
        elif 'disconnected' in x:
            if 'Client' not in name:
                logged[name]=0
    f.close()
    log.reverse()
    log.sort(key=fctSort)
    
    lastName=""
    name=""
    i=0
    for x in log:
        e=x.split(":")
        name=e[3].strip()
        if name != lastName:
            prov[name]=e[4][15:]
            lastName=name
            i+=1
    readbip()
    for x in bip:
        if x not in prov:
            prov[x]="---.---.---.---"
    i=0
    for key in sorted(prov.keys()) :
        links.append(key)
        ips.append(prov[key])
        i+=1


def refreshPad(pad):
    for y in range(len(links)):
      wrline(pad,0,y)



def draw_menu(stdscr):
    """ programme principal.

        - affichage à l'écran
        - lecture clavier

    """
    global H1,MH,W1,MW1

    k = 0
    cursor_x = 0
    cursor_y = 0

    stdscr.keypad(True)

    HScreen, WScreen = stdscr.getmaxyx()
    # test écran
#    stdscr.addstr("H = {}  -  W = {}".format(HScreen,WScreen))
#    stdscr.refresh()
#    c = stdscr.getch()
    stdscr.clear()

    stdscr.refresh()

    # Start colors in curses
    curses.start_color()
    curses.init_pair(1, curses.COLOR_BLUE, curses.COLOR_WHITE)
    curses.init_pair(2, curses.COLOR_RED, curses.COLOR_WHITE)
    curses.init_pair(3, curses.COLOR_BLUE, curses.COLOR_YELLOW)
    curses.init_pair(4, curses.COLOR_RED, curses.COLOR_YELLOW)
    curses.init_pair(5, curses.COLOR_WHITE, curses.COLOR_BLUE)
    curses.init_pair(6, curses.COLOR_GREEN, curses.COLOR_WHITE)
    curses.init_pair(7, curses.COLOR_WHITE, curses.COLOR_GREEN)


    readlog()

    if WScreen >= 144:
        TypeScreen = LargeScreen
        H1=HScreen-6
        MH=5
        W1=94
        MW1=(WScreen-144)//2
    elif WScreen >= 95:
        TypeScreen = MediumScreen
        H1=HScreen-6
        MH=5
        W1=94
        MW1=(WScreen-95)//2
    else:
        TypeScreen = SmallScreen
        H1=HScreen-6
        MH=5
        W1=51
        MW1=(WScreen-50)//2

    
    win=curses.newwin(H1,W1,MH,MW1)
    win.bkgd(curses.color_pair(0)|curses.A_REVERSE)
    Hw,Ww = win.getmaxyx()
    titre= "FIREWALL RRF - BLOCAGE DEBLOCAGE DE NODES"
    Xw= (Ww - len(titre))//2
    win.border()
    win.addstr(0,Xw,titre)
    win.refresh()
    pad = curses.newpad(len(links), 92)
    pad.bkgd(curses.color_pair(1))

    if TypeScreen == LargeScreen:
        win2=curses.newwin(Hw,50,MH,MW1+W1)
        win2.bkgd(curses.color_pair(1))
        win2.border()
        Hw,Ww = win2.getmaxyx()
        titre= " NODES DE MEME IP  "
        Xw= (Ww - len(titre))//2
        win2.addstr(0,Xw,titre)
        win2.refresh()
#    pan2 = curses.panel.new_panel(win2)

#    time.sleep(10)
#    pan2.hide()

#    subw=win.subwin(20,50,8,150)

#    subwOpen(win)

#    for y in range(len(links)):
#      wrline(pad,0,y)
    refreshPad(pad)
    wrline(pad,1,0) 
    

    commandes(stdscr)

    numero=0
    topscr=0
    pad.refresh( topscr,0, MH+2,MW1+2, MH+H1-2, MW1+W1-2)

    if TypeScreen == LargeScreen:
        fillWin2(win2,links[0],ips[0],curses.color_pair(1))

    while True:
        c = stdscr.getch()
        lastnum=numero
        if c == 27:
            # Touche ESC
            exit()
        elif c == curses.KEY_DOWN:
            numero+=1
            if numero>=len(links):
                numero=numero-1
            if numero-topscr>=H1-3:
                topscr+=1
        elif c == curses.KEY_UP:
            if numero >0:
                numero=numero-1
            if numero < topscr:
                topscr = numero
        elif c == curses.KEY_PPAGE:
            numero=numero-H1+3
            if numero < 0:
                numero = 0
            if numero < topscr:
                topscr = numero
        elif c == curses.KEY_NPAGE:
            numero = numero+H1-3
            if numero >= len(links):
                numero = len(links)-1
            topscr=numero
            if numero+H1-3 >= len(links):
                topscr=len(links)-H1+3
        elif c == 269:
            # Touche F5 : Bloque un Node
            name = links[numero]
            nodeIP = ips[numero]
            if nodeIP not in bip.values():
                if name in bip:
                    delDrop(bip[name],name)
                bip[name]=nodeIP
                addDrop(nodeIP,name)
                writebip()
                refreshPad(pad)
        elif c == 270:
            # Touche F6 : Autorise un Node
            name = links[numero]
            nodeIP = ips[numero]
            if name in bip:
                delDrop(bip[name],name)
                del bip[name]
                writebip()
            else:
                name = getNameBip(nodeIP)
                if name != False:
                    delDrop(bip[name],name)
                    del bip[name]
                    writebip()
            refreshPad(pad)
#        elif c == curses.KEY_HOME:
#            numero = 0
#            topscr = 0
        
        wrline(pad,0,lastnum)
        wrline(pad,1,numero)
        pad.refresh( topscr,0, MH+2,MW1+2, MH+H1-2, MW1+W1-2)

#        pad.refresh(topscr,0,7,7, HScreen-10, WScreen-10)
        if TypeScreen == LargeScreen:
            fillWin2(win2,links[numero],ips[numero], curses.color_pair(1))
#        win2.refresh()



curses.wrapper(draw_menu)
